import { createRef, MouseEventHandler, useEffect, useState } from "react";
import { useSession } from "./Session";

interface ContextMenuItem {
  label: string;
  onClick: () => void;
}

export default function Avatar() {
  const { user, onLogout } = useSession();
  const [contextMenu, setContextMenu] = useState<{
    open: boolean;
    allowOpen: boolean;
    mouseX: number;
    mouseY: number;
  }>({ open: false, allowOpen: true, mouseX: 0, mouseY: 0 });
  const contextMenuRef = createRef<HTMLUListElement>();
  const avatarRef = createRef<HTMLDivElement>();

  const contextMenuItems: ContextMenuItem[] = [
    { label: "View Profile", onClick: handleViewProfile },
    { label: "Logout", onClick: onLogout },
  ];

  const handleMenuClick: MouseEventHandler<HTMLDivElement> = (event) => {
    if (!contextMenu.allowOpen) return;
    event.preventDefault();
    setContextMenu({
      open: !contextMenu.open,
      allowOpen: false,
      mouseX: event.clientX + 4,
      mouseY: event.clientY + 2,
    });
  };

  useEffect(() => {
    if (contextMenu.open) {
      document.addEventListener("click", handleCloseContextMenuOutside);
    }
    return () => {
      document.removeEventListener("click", handleCloseContextMenuOutside);
    };
  }, [contextMenu.open]);

  const handleMenuClose = () => {
    setContextMenu({ ...contextMenu, open: false });
  };

  const handleCloseContextMenuOutside: (event: MouseEvent) => void = (ev) => {
    if (
      !(
        contextMenuRef.current?.contains(ev.target as Node) ||
        avatarRef.current?.contains(ev.target as Node)
      )
    ) {
      handleMenuClose();
    }
  };

  const [dialog, setDialog] = useState(<></>);
  const dialogRef = createRef<HTMLDialogElement>();

  function handleViewProfile() {
    handleMenuClose();
    setContextMenu({ ...contextMenu, allowOpen: false });
    dialogRef.current?.showModal();
    setDialog(
      <ul style={{ listStyle: "none" }}>
        <li>username: {user?.username}</li>
      </ul>
    );
  }

  return (
    <div
      className="avatar"
      onContextMenu={handleMenuClick}
      style={{ display: user ? "block" : "none" }}
      onClick={handleMenuClick}
      ref={avatarRef}
    >
      👤 {user?.username}
      {contextMenu.open && (
        <ul
          className="context-menu"
          ref={contextMenuRef}
          style={{
            zIndex: 3,
            position: "absolute",
            top: contextMenu.mouseY,
            left: contextMenu.mouseX,
            background: "white",
            border: "1px solid #ddd",
            padding: 0,
            margin: 0,
            listStyle: "none",
          }}
        >
          {contextMenuItems.map((item, index) => (
            <li
              key={index}
              style={{
                padding: "10px",
                borderBottom: "1px solid #ddd",
                cursor: "pointer",
              }}
              onClick={() => {
                item.onClick();
                handleMenuClose();
              }}
            >
              {item.label}
            </li>
          ))}
        </ul>
      )}
      <dialog
        id="AvatarDialog"
        ref={dialogRef}
        onClick={(event) => {
          event.currentTarget.close();
          setContextMenu({ ...contextMenu, allowOpen: true });
        }}
      >
        {dialog}
      </dialog>
    </div>
  );
}
